/*
 *   Java-Gnome
 *
 *   Copyright (C) 1998 Olivier Gutknecht <gutkneco@lirmm.fr>
 *
 *   The Java-Gnome bindings library is free software distributed under
 *   the terms of the GNU Library General Public License version 2 
 *
 ***********************************************************************/

#include <jni.h>
#include <gtk/gtk.h>

// All the possible classes we are interesed in
static jclass booleanClass = NULL;
static jclass byteClass = NULL;
static jclass charClass = NULL;
static jclass doubleClass = NULL;
static jclass floatClass = NULL;
static jclass intClass = NULL;
static jclass longClass = NULL;
static jclass shortClass = NULL;
static jclass stringClass = NULL;

// The fieldID fo if the object is a GTK object
static jfieldID gtkobj_fieldID = NULL;

gpointer
jobject_to_gpointer (JNIEnv *env, jobject obj)
{
  jclass obj_class;
  jmethodID obj_methodID;
  gpointer result;

  // This may take very long.  Is there a more elegant way?
  if (!booleanClass)
    booleanClass = (*env)->FindClass(env, "java.lang.Boolean");
  if (!byteClass)
    byteClass = (*env)->FindClass(env, "java.lang.Byte");
  if (!charClass)
    charClass = (*env)->FindClass(env, "java.lang.Char");
  if (!doubleClass)
    doubleClass = (*env)->FindClass(env, "java.lang.Double");
  if (!floatClass)
    floatClass = (*env)->FindClass(env, "java.lang.Float");
  if (!intClass)
    intClass = (*env)->FindClass(env, "java.lang.Integer");
  if (!longClass)
    longClass = (*env)->FindClass(env, "java.lang.Long");
  if (!shortClass)
    shortClass = (*env)->FindClass(env, "java.lang.Short");
  if (!stringClass)
    stringClass = (*env)->FindClass(env, "java.lang.String");

  obj_class = (*env)->GetObjectClass(env, obj);

  // Determine the class and return a pointer to it
  if (obj_class == booleanClass)
    {
      result = g_malloc(sizeof(int));
      obj_methodID = (*env)->GetMethodID(env, obj, "booleanValue", "()Z");
      *((int*) result) = (int) (*env)->CallBooleanMethod(env, obj, obj_methodID);
    }
  else if (obj_class == byteClass)
    {
      result = g_malloc(sizeof(int));
      obj_methodID = (*env)->GetMethodID(env, obj, "byteValue", "()B");
      *((int*) result) = (int) (*env)->CallByteMethod(env, obj, obj_methodID);
    }
  else if (obj_class == charClass)
    {
      result = g_malloc(sizeof(char));
      obj_methodID = (*env)->GetMethodID(env, obj, "charValue", "()C");
      *((char*) result) = (char) (*env)->CallByteMethod(env, obj, obj_methodID);
    }
  else if (obj_class == doubleClass)
    {
      result = g_malloc(sizeof(double));
      obj_methodID = (*env)->GetMethodID(env, obj, "doubleValue", "()D");
      *((double*) result) = (double) (*env)->CallDoubleMethod(env, obj, obj_methodID);
    }
  else if (obj_class == floatClass)
    {
      result = g_malloc(sizeof(float));
      obj_methodID = (*env)->GetMethodID(env, obj, "floatValue", "()F");
      *((float*) result) = (float) (*env)->CallFloatMethod(env, obj, obj_methodID);
    }
  else if (obj_class == intClass)
    {
      result = g_malloc(sizeof(int));
      obj_methodID = (*env)->GetMethodID(env, obj, "intValue", "()I");
      *((int*) result) = (int) (*env)->CallIntMethod(env, obj, obj_methodID);
    }
  else if (obj_class == longClass)
    {
      result = g_malloc(sizeof(long));
      obj_methodID = (*env)->GetMethodID(env, obj, "longValue", "()J");
      *((long*) result) = (long) (*env)->CallLongMethod(env, obj, obj_methodID);
    }
  else if (obj_class == shortClass)
    {
      result = g_malloc(sizeof(short));
      obj_methodID = (*env)->GetMethodID(env, obj, "shortValue", "()S");
      *((short*) result) = (short) (*env)->CallShortMethod(env, obj, obj_methodID);
    }
  else if (obj_class == stringClass)
    {
      result = (*env)->GetStringUTFChars(env, obj, 0);
    }
  else // Currently just assume it is a GTKObject
    {
      // Copied blatently from jobject_to_gtk -> written by sopwith/olg
      jint obj_pointer;
      if(!gtkobj_fieldID)
	gtkobj_fieldID = (*env)->GetFieldID(env, (*env)->GetObjectClass(env, obj), "nativepeer", "J");
      obj_pointer = (*env)->GetIntField(env, obj, gtkobj_fieldID);
      return GTK_OBJECT((void *)obj_pointer);
    }

  return result;
}
